<?php
// Include tenant-specific config if it exists
if(file_exists('../../'.$_REQUEST['tenant'].'/config.php')) {
    require_once('../../'.$_REQUEST['tenant'].'/config.php');
} elseif(file_exists('/config.php')) {
    require_once('/config.php');
}
use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\Exception;
//echo '../output/'.$_REQUEST['source_1'].'/'.$_REQUEST['attachment_1'];

require '../packages/mailer/Exception.php';
require '../packages/mailer/PHPMailer.php';
require '../packages/mailer/SMTP.php';

// Retrieve POST data
$email = $_POST['email'] ?? '';
$type = $_POST['type'] ?? '';

// Validate the email and user type
if (!filter_var($email, FILTER_VALIDATE_EMAIL) || empty($type)) {
    echo 'error';
    exit();
}

// Sanitize the table name (user type) to prevent SQL injection
$allowed_types = ['surveyor', 'contractors', 'admins'];
if (!in_array($type, $allowed_types)) {
    echo 'error';
    exit();
}

// Dynamically reference the table based on the user type
$table = $type;

// Check if the user exists in the dynamically referenced table
$stmt = $con->prepare("SELECT id FROM $table WHERE email = ?");
if ($stmt) {
    $stmt->bind_param('s', $email);
    $stmt->execute();
    $result = $stmt->get_result();
    $user = $result->fetch_assoc();
    $stmt->close();
} else {
    echo 'Error preparing statement: ' . $con->error;
    exit();
}

// If user exists, generate and store token
if ($user) {
    $token = bin2hex(random_bytes(32)); // Generate a unique token
    $expiry = date('Y-m-d H:i:s', strtotime('+2 hour')); // Token expires in 1 hour

    // Insert the token into the password_resets table
    $stmt = $con->prepare("INSERT INTO password_resets (email, token, expires_at, user_type) VALUES (?, ?, ?, ?)");
    if ($stmt) {
        $stmt->bind_param('ssss', $email, $token, $expiry, $type);
        $stmt->execute();
        $stmt->close();
    } else {
        echo 'Error preparing statement: ' . $con->error;
        exit();
    }

    // Create the password reset link
    function encryptResetLinkData($data) {
        $encryption_key = 'Leekylecallum197!'; // Store securely
        $iv = openssl_random_pseudo_bytes(openssl_cipher_iv_length('aes-256-cbc'));
        $encrypted = openssl_encrypt($data, 'aes-256-cbc', $encryption_key, 0, $iv);
        return base64_encode($encrypted . '::' . $iv);
    }
    
    $resetData = json_encode([
        'token' => $token,
        'email' => $email,
        'type' => $type
    ]);
    
    $encryptedData = encryptResetLinkData($resetData);
    $resetLink = $systemUser . "/reset_password.php?data=" . urlencode($encryptedData);
    

    // Prepare and send the email
    $subject = getCompanyDetails('name') . "- Password Reset";
    $header = file_get_contents('../includes/master/email_template.php');
    $message = "Please click the following link to reset your password: <a href='https://$resetLink'>RESET</a>";
    
    $header = str_replace('[MESSAGE]', nl2br($message), $header);
    $header = str_replace('[LOGO]', getCompanyDetails('web').'/images/company/logo.jpg?'.rand(), $header);
    $header = str_replace('[ENREP]', getCompanyDetails('web').'/images/logo_border.png?'.rand(), $header);
    $header = str_replace('[ADDRESS]', getCompanyDetails('address'), $header);
    $header = str_replace('[CONTACT]', getCompanyDetails('contact'), $header);
    $header = str_replace('[SUBJECT]', $subject, $header);

    $mail = new PHPMailer(true);
    try {
        $mail->isSMTP();
        $mail->Host = getEmailSystemSettings('host');
        $mail->SMTPAuth = true;
        $mail->Username = getEmailSystemSettings('username');
        $mail->Password = getEmailSystemSettings('password');
        $mail->SMTPSecure = 'spa';
        $mail->Port = 587;
        $mail->setFrom(getEmailSystemSettings('from'), getEmailSystemSettings('from_name'));
        $mail->addAddress($email);
        $mail->isHTML(true);
        $mail->Subject = $subject;
        $mail->Body    = $header;
        $mail->AltBody = $header;

        $mail->send();
        echo "success";
    } catch (Exception $e) {
        echo 'error';
    }
} else {
    echo 'error';
}
?>