<?php 
// Include tenant-specific config if it exists
if (file_exists('../../'.$_REQUEST['tenant'].'/config.php')) {
    require_once('../../'.$_REQUEST['tenant'].'/config.php');
} elseif (file_exists('/config.php')) {
    require_once('/config.php');
}

// Handle password update
$token = $_POST['token'];
$email = $_POST['email'];
$newPassword = $_POST['new_password'];
$confirmPassword = $_POST['confirm_password'];
$type = $_POST['type']; // User type: surveyor, contractor, admin

// Validate passwords
if ($newPassword !== $confirmPassword) {
    echo json_encode(['status' => 'error', 'message' => 'Passwords do not match']);
    exit();
}

// Check if the token is valid and has not expired
$stmt = $con->prepare("SELECT * FROM password_resets WHERE email = ? AND token = ? AND expires_at > NOW() AND user_type = ?");
$stmt->bind_param('sss', $email, $token, $type);
$stmt->execute();
$result = $stmt->get_result();
$resetRequest = $result->fetch_assoc();

if (!$resetRequest) {
    echo json_encode(['status' => 'error', 'message' => 'Invalid or expired token']);
    exit();
}

// Hash the new password
$hashedPassword = password_hash($newPassword, PASSWORD_DEFAULT);

// Update the password in the respective user table
$stmt = $con->prepare("UPDATE $type SET password = AES_ENCRYPT(?, ?) WHERE email = ?");
if ($stmt) {
    $stmt->bind_param('sss', $newPassword, $email, $email);
    $stmt->execute();
    $stmt->close();
} else {
    echo json_encode(['status' => 'error', 'message' => 'Error updating password: ' . $con->error]);
    exit();
}

// Delete the reset token
$stmt = $con->prepare("DELETE FROM password_resets WHERE email = ?");
$stmt->bind_param('s', $email);
$stmt->execute();
$stmt->close();

echo json_encode(['status' => 'success']);
$con->close();
?>