(function (factory) {
  if (typeof define === "function" && define.amd) {
    // AMD. Register as anonymous module.
    define(["jquery"], factory);
  } else if (typeof exports === "object") {
    // Node / CommonJS
    factory(require_once("jquery"));
  } else {
    // Browser globals.
    factory(jQuery);
  }
})(function ($) {
  "use strict";

  var console = window.console || { log: function () {} };

  function Croplev2($element) {
    this.$container = $element;

    this.$lev2View = this.$container.find("#lev2-view");
    this.$lev2 = this.$lev2View.find("img");
    this.$lev2Modal = this.$container.find("#lev2-modal");
    this.$loading = this.$container.find(".loading");

    this.$lev2Form = this.$lev2Modal.find("#lev2-form");
    this.$lev2Upload = this.$lev2Form.find(".lev-upload");
    this.$lev2Src = this.$lev2Form.find(".lev-src");
    this.$lev2Data = this.$lev2Form.find(".lev-data");
    this.$lev2Input = this.$lev2Form.find("#lev2-input");
    this.$lev2Save = this.$lev2Form.find(".lev-save");
    this.$lev2Btns = this.$lev2Form.find(".lev-btns");
    this.$lev2Ratio = this.$lev2Form.find(".lev-ratio").val();

    this.$lev2Wrapper = this.$lev2Modal.find(".lev-wrapper");
    this.$lev2Preview = this.$lev2Modal.find(".lev-preview");

    this.init();
  }

  Croplev2.prototype = {
    constructor: Croplev2,

    support: {
      fileList: !!$('<input type="file">').prop("files"),
      blobURLs: !!window.URL && URL.createObjectURL,
      formData: !!window.FormData,
    },

    init: function () {
      this.support.datauri = this.support.fileList && this.support.blobURLs;

      if (!this.support.formData) {
        this.initIframe();
      }

      this.initTooltip();
      this.initModal();
      this.addListener();
    },

    addListener: function () {
      this.$lev2View.on("click", $.proxy(this.click, this));
      this.$lev2Input.on("change", $.proxy(this.change, this));
      this.$lev2Form.on("submit", $.proxy(this.submit, this));
      this.$lev2Btns.on("click", $.proxy(this.rotate, this));
    },

    initTooltip: function () {
      this.$lev2View.tooltip({
        placement: "bottom",
      });
    },

    initModal: function () {
      this.$lev2Modal.modal({
        show: false,
      });
    },

    initPreview: function () {
      var url = this.$lev2.attr("src");

      this.$lev2Preview.html('<img src="' + url + '">');
    },

    initIframe: function () {
      var target = "upload-iframe-" + new Date().getTime();
      var $iframe = $("<iframe>").attr({
        name: target,
        src: "",
      });
      var _this = this;

      // Ready ifrmae
      $iframe.one("load", function () {
        // respond response
        $iframe.on("load", function () {
          var data;

          try {
            data = $(this).contents().find("body").text();
          } catch (e) {
            console.log(e.message);
          }

          if (data) {
            try {
              data = $.parseJSON(data);
            } catch (e) {
              console.log(e.message);
            }

            _this.submitDone(data);
          } else {
            _this.submitFail("Image upload failed!");
          }

          _this.submitEnd();
        });
      });

      this.$iframe = $iframe;
      this.$lev2Form.attr("target", target).after($iframe.hide());
    },

    click: function () {
      this.$lev2Modal.modal("show");
      this.initPreview();
    },

    change: function () {
      var files;
      var file;

      if (this.support.datauri) {
        files = this.$lev2Input.prop("files");

        if (files.length > 0) {
          file = files[0];

          if (this.isImageFile(file)) {
            if (this.url) {
              URL.revokeObjectURL(this.url); // Revoke the old one
            }

            this.url = URL.createObjectURL(file);
            this.startCropper();
          }
        }
      } else {
        file = this.$lev2Input.val();

        if (this.isImageFile(file)) {
          this.syncUpload();
        }
      }
    },

    submit: function () {
      if (!this.$lev2Src.val() && !this.$lev2Input.val()) {
        return false;
      }

      if (this.support.formData) {
        this.ajaxUpload();
        return false;
      }
    },

    rotate: function (e) {
      var data;

      if (this.active) {
        data = $(e.target).data();

        if (data.method) {
          this.$img.cropper(data.method, data.option);
        }
      }
    },

    isImageFile: function (file) {
      if (file.type) {
        return /^image\/\w+$/.test(file.type);
      } else {
        return /\.(jpg|jpeg|png|gif)$/.test(file);
      }
    },

    startCropper: function () {
      var _this = this;
      var ratio = this.$lev2Ratio.split("|");
      //alert(ratio[0])
      if (this.active) {
        this.$img.cropper("replace", this.url);
      } else {
        this.$img = $('<img src="' + this.url + '">');
        this.$lev2Wrapper.empty().html(this.$img);
        this.$img.cropper({
          aspectRatio: ratio[0] / ratio[1],
          preview: this.$lev2Preview.selector,
          crop: function (e) {
            var json = ['{"x":' + e.x, '"y":' + e.y, '"height":' + e.height, '"width":' + e.width, '"rotate":' + e.rotate + "}"].join();

            _this.$lev2Data.val(json);
          },
        });

        this.active = true;
      }

      this.$lev2Modal.one("hidden.bs.modal", function () {
        _this.$lev2Preview.empty();
        _this.stopCropper();
      });
    },

    stopCropper: function () {
      if (this.active) {
        this.$img.cropper("destroy");
        this.$img.remove();
        this.active = false;
      }
    },

    ajaxUpload: function () {
      var url = this.$lev2Form.attr("action");
      var data = new FormData(this.$lev2Form[0]);
      var _this = this;

      $.ajax(url, {
        type: "post",
        data: data,
        dataType: "json",
        processData: false,
        contentType: false,

        beforeSend: function () {
          _this.submitStart();
        },

        success: function (data) {
          _this.submitDone(data);
        },

        error: function (XMLHttpRequest, textStatus, errorThrown) {
          _this.submitFail(textStatus || errorThrown);
        },

        complete: function (datax) {
          console.log(datax);
          _this.submitEnd();
        },
      });
    },

    syncUpload: function () {
      this.$lev2Save.click();
    },

    submitStart: function () {
      this.$loading.fadeIn();
    },

    submitDone: function (data) {
      console.log(data);

      if ($.isPlainObject(data) && data.state === 200) {
        if (data.result) {
          this.url = data.result;

          if (this.support.datauri || this.uploaded) {
            this.uploaded = false;
            this.cropDone();
          } else {
            this.uploaded = true;
            this.$lev2Src.val(this.url);
            this.startCropper();
          }

          this.$lev2Input.val("");
        } else if (data.message) {
          this.alert(data.message);
        }
      } else {
        this.alert("Failed to response");
      }
    },

    submitFail: function (msg) {
      this.alert(msg);
    },

    submitEnd: function () {
      this.$loading.fadeOut();
    },

    cropDone: function () {
      this.$lev2Form.get(0).reset();
      this.$lev2.attr("src", this.url);
      this.stopCropper();
      this.$lev2Modal.modal("hide");
    },

    alert: function (msg) {
      var $alert = ['<div class="alert alert-danger lev2-alert alert-dismissable">', '<button type="button" class="close" data-dismiss="alert">&times;</button>', msg, "</div>"].join("");

      this.$lev2Upload.after($alert);
    },
  };

  $(function () {
    return new Croplev2($("#crop-lev2"));
  });
});
