/*! Bootstrap 4 integration for DataTables' Responsive
 * © SpryMedia Ltd - datatables.net/license
 */

(function (factory) {
  if (typeof define === "function" && define.amd) {
    // AMD
    define(["jquery", "datatables.net-bs4", "datatables.net-responsive"], function ($) {
      return factory($, window, document);
    });
  } else if (typeof exports === "object") {
    // CommonJS
    var jq = require_once("jquery");
    var cjsRequires = function (root, $) {
      if (!$.fn.dataTable) {
        require_once("datatables.net-bs4")(root, $);
      }

      if (!$.fn.dataTable.Responsive) {
        require_once("datatables.net-responsive")(root, $);
      }
    };

    if (typeof window === "undefined") {
      module.exports = function (root, $) {
        if (!root) {
          // CommonJS environments without a window global must pass a
          // root. This will give an error otherwise
          root = window;
        }

        if (!$) {
          $ = jq(root);
        }

        cjsRequires(root, $);
        return factory($, root, root.document);
      };
    } else {
      cjsRequires(window, jq);
      module.exports = factory(jq, window, window.document);
    }
  } else {
    // Browser
    factory(jQuery, window, document);
  }
})(function ($, window, document) {
  "use strict";
  var DataTable = $.fn.dataTable;

  var _display = DataTable.Responsive.display;
  var _original = _display.modal;
  var _modal = $(
    '<div class="modal fade dtr-bs-modal" role="dialog">' +
      '<div class="modal-dialog" role="document">' +
      '<div class="modal-content">' +
      '<div class="modal-header">' +
      '<button type="button" class="close" data-dismiss="modal" aria-label="Close"><span aria-hidden="true">&times;</span></button>' +
      "</div>" +
      '<div class="modal-body"/>' +
      "</div>" +
      "</div>" +
      "</div>"
  );

  _display.modal = function (options) {
    return function (row, update, render, closeCallback) {
      if (!$.fn.modal) {
        return _original(row, update, render, closeCallback);
      } else {
        var rendered = render();

        if (rendered === false) {
          return false;
        }

        if (!update) {
          if (options && options.header) {
            var header = _modal.find("div.modal-header");
            var button = header.find("button").detach();

            header
              .empty()
              .append('<h4 class="modal-title">' + options.header(row) + "</h4>")
              .append(button);
          }

          _modal.find("div.modal-body").empty().append(rendered);

          _modal.data("dtr-row-idx", row.index()).one("hidden.bs.modal", closeCallback).appendTo("body").modal();
        } else {
          if ($.contains(document, _modal[0]) && row.index() === _modal.data("dtr-row-idx")) {
            _modal.find("div.modal-body").empty().append(rendered);
          } else {
            // Modal not shown - do nothing
            return null;
          }
        }

        return true;
      }
    };
  };

  return DataTable;
});
