function printContainer(input, options = {}) {
  const defaultOptions = {
    orientation: "portrait", // 'portrait' or 'landscape'
    header: "", // HTML string for header content
    footer: "", // HTML string for footer content
    logoUrl: "", // URL of the logo image
    styles: [], // Array of additional local stylesheet URLs
    headerHeight: 60, // Height of the header in px
    footerHeight: 40, // Height of the footer in px
    margin: 0, // Page margins in mm
  };

  const settings = { ...defaultOptions, ...options };

  let contentHtml;
  if (document.getElementById(input)) {
    const container = document.getElementById(input);
    contentHtml = container.innerHTML;
  } else {
    contentHtml = input; // Assume input is HTML content
  }

  const pageHeight = settings.orientation === "landscape" ? 210 : 297; // A4 page height in mm
  const availableContentHeight = pageHeight - (settings.headerHeight + settings.footerHeight + 2 * settings.margin);

  const iframe = document.createElement("iframe");
  document.body.appendChild(iframe);

  iframe.style.position = "absolute";
  iframe.style.width = "0";
  iframe.style.height = "0";
  iframe.style.border = "none";

  const iframeDoc = iframe.contentDocument || iframe.contentWindow.document;

  iframeDoc.open();
  iframeDoc.write(`
    <!DOCTYPE html>
    <html lang="en">
    <head>
      <meta charset="UTF-8">
      <meta name="viewport" content="width=device-width, initial-scale=1.0">
      <title>Print Preview</title>
      ${settings.styles.map((styleUrl) => `<link rel="stylesheet" href="${styleUrl}">`).join("")}
      <style>
        @media print {
          body {
            margin: 0;
            padding: ${settings.margin}mm;
            box-sizing: border-box;
            display: flex;
            flex-direction: column;
            height: 100vh;
          }
          .page {
            display: flex;
            flex-direction: column;
            height: 100%;
            page-break-after: always;
            position: relative;
          }
          .header {
            height: ${settings.headerHeight}px;
            text-align: center;
            background: #fff;
            display: flex;
            align-items: center;
            justify-content: center;
          }
          .content {
            flex: 1;
            margin: 20px;
            overflow: auto;
            position: relative;
          }
          .footer {
            height: ${settings.footerHeight}px;
            text-align: center;
            color: #ccc;
            background: #fff;
            display: flex;
            align-items: center;
            justify-content: center;
            position: absolute;
            bottom: 0;
            left: 0;
            right: 0;
            width: 100%;
          }
          .logo {
            position: absolute;
            top: 10px;
            left: 10px;
            max-width: 100px;
            max-height: 50px;
            z-index: 1000;
          }
          table {
            width: 100%;
            border-collapse: collapse;
            page-break-inside: auto;
          }
          tr {
            page-break-inside: avoid;
            page-break-after: auto;
          }
          thead {
            display: table-header-group;
          }
          tfoot {
            display: table-footer-group;
          }
          th, td {
            padding: 8px;
            border: 1px solid #ddd;
          }
          @page {
            size: ${settings.orientation === "landscape" ? "landscape" : "portrait"};
            margin: 0;
          }
        }
        @media print {
          body {
            -webkit-print-color-adjust: exact;
          }
        }
        span.noprint, div.noprint, a.noprint {
            display: none !important;
        }
      </style>
    </head>
    <body>
      <div class="page">
        ${settings.header ? `<div class="header">${settings.logoUrl ? `<img src="${settings.logoUrl}" alt="Logo" class="logo">` : ""}${settings.header}</div>` : ""}
        <div class="content">${contentHtml}</div>
        ${settings.footer ? `<div class="footer">${settings.footer}</div>` : ""}
      </div>
      <script>
        const noprintElements = document.querySelectorAll('.noprint');
        noprintElements.forEach(el => el.style.display = 'none');
        window.onload = function() {
          window.print();
          window.onafterprint = function() {
            parent.document.body.removeChild(parent.document.querySelector('iframe'));
          };
        };
      </script>
    </body>
    </html>
  `);

  iframeDoc.close();
}

function removeNoPrintElements(doc) {
  const elements = doc.querySelectorAll(".noprint");
  elements.forEach((el) => (el.style.display = "none"));
}
